/* d88image.h version 1.2.5 */
#ifndef __D88IMAGE_H__
#define __D88IMAGE_H__

#define D88_TRACK_COUNT				(164)
#define D88_TRACK_COUNT_OLD			(160)
#define D88_DISK_NAME_SIZE			(17)
#define D88_DISK_RESERVED_SIZE		(9)
#define D88_SECTOR_RESERVED_SIZE	(5)

#define D88_DISK_HEADER_SIZE		(0x2B0)
#define D88_DISK_HEADER_SIZE_OLD	(0x2A0)
#define D88_SECTOR_HEADER_SIZE		(0x10)

#define D88_DISK_TYPE_2D			(0x00)
#define D88_DISK_TYPE_2DD			(0x10)
#define D88_DISK_TYPE_2HD			(0x20)

#define D88_FORMAT_CUSTOM			(-2)
#define D88_FORMAT_UNKNOWN			(-1)
#define D88_FORMAT_2D				(0)		/* disk_type=0x00 C/H/R/N=0x00-0x27/0x00-0x01/0x01-0x10/0x01 num_sectors=16 density=0x00 */
#define D88_FORMAT_2DD_8			(1)		/* disk_type=0x10 C/H/R/N=0x00-0x4F/0x00-0x01/0x01-0x08/0x02 num_sectors=8  density=0x00 */
#define D88_FORMAT_2DD_9			(2)		/* disk_type=0x10 C/H/R/N=0x00-0x4F/0x00-0x01/0x01-0x09/0x02 num_sectors=9  density=0x00 */
#define D88_FORMAT_2HD				(3)		/* disk_type=0x20 C/H/R/N=0x00-0x4C/0x00-0x01/0x01-0x08/0x03 num_sectors=8  density=0x00 */
#define D88_FORMAT_2HDA				(4)		/* disk_type=0x20 C/H/R/N=0x00-0x4F/0x00-0x01/0x01-0x08/0x03 num_sectors=8  density=0x00 */
#define D88_FORMAT_2HS				(5)		/* disk_type=0x20 C/H/R/N=0x00-0x4F/0x00-0x01/0x0A-0x12/0x03 num_sectors=9  density=0x00 */
											/*      sector[0] C/H/R/N=0x00     /0x00     /0x01     /0x03                             */
#define D88_FORMAT_2HDE				(6)		/* disk_type=0x20 C/H/R/N=0x00-0x4F/0x80-0x81/0x01-0x09/0x03 num_sectors=9  density=0x00 */
											/*      sector[0] C/H/R/N=0x00     /0x00     /0x01     /0x03                             */
#define D88_FORMAT_2HC				(7)		/* disk_type=0x20 C/H/R/N=0x00-0x4F/0x00-0x01/0x01-0x0F/0x02 num_sectors=15 density=0x00 */
#define D88_FORMAT_2HQ				(8)		/* disk_type=0x20 C/H/R/N=0x00-0x4F/0x00-0x01/0x01-0x12/0x02 num_sectors=18 density=0x00 */
#define D88_FORMAT_OS9				(9)		/* disk_type=0x20 C/H/R/N=0x00-0x4C/0x00-0x01/0x01-0x1A/0x01 num_sectors=26 density=0x00 */



struct d88_sector_t {
	/* 0x10 bytes */
	unsigned char C;					/* cylinder no */
	unsigned char H;					/* head no 0x00=front, 0x01=back */
	unsigned char R;					/* record no */
	unsigned char N;					/* record length 0x01-0x06=256-8192bytes (256 x 2^(N-1)bytes) */
	short num_sectors;					/* 2bytes */
	unsigned char density;				/* 0x00=double, 0x40=single */
	unsigned char deleted_data;			/* 0x00=normal, 0x10=deleted data */
	unsigned char status;				/* 0x00=normal, else=disk bios status */
	char reserved[D88_SECTOR_RESERVED_SIZE];
	short sector_size;					/* 2bytes */

	unsigned char *data;
};

struct d88_track_t {
	int num_sectors;
	struct d88_sector_t *sectors;
};

struct d88_disk_t {
	struct d88_disk_t *next;

	/* 0x2B0 bytes */
	char name[D88_DISK_NAME_SIZE];	   	/* 16bytes + 0x00 (zero fill) */
	char reserved[D88_DISK_RESERVED_SIZE];
	unsigned char write_protect_flag;	/* 0x00=off, 0x10=write protected */
	unsigned char disk_type;			/* 0x00=2D, 0x10=2DD, 0x20=2HD */
	int disk_size;						/* 4bytes */
	int track_offset[D88_TRACK_COUNT];	/* 4bytes x 164 */

	struct d88_track_t tracks[D88_TRACK_COUNT];
};

struct d88_image_t {
	int num_disks;
	struct d88_disk_t *disks;
};

extern void free_d88_disk(struct d88_disk_t *disk, int free_all);
extern void free_d88_image(struct d88_image_t *image, int free_all);
extern struct d88_disk_t *get_d88_disk(struct d88_image_t *image, int index);
extern int read_d88_image(struct d88_image_t *image, const char *name);
extern int read_d88_image_from_buffer(struct d88_image_t *image, const unsigned char *data, int data_size);
extern int read_d88_disk_from_buffer(struct d88_disk_t *disk, const unsigned char *data, int data_size);
extern int write_d88_image(const struct d88_image_t *image, const char *name);
extern int write_d88_disk(const struct d88_disk_t *disk, const char *name);
extern int write_d88_image_to_buffer(const struct d88_image_t *image, unsigned char **data, int *data_size);
extern int write_d88_disk_to_buffer(const struct d88_disk_t *disk, unsigned char *data, unsigned char **end, int data_size);
extern struct d88_image_t *create_d88_empty_image(void);
extern struct d88_disk_t *create_d88_disk(const char *name, int format);
extern struct d88_sector_t *create_d88_sectors(unsigned char C, unsigned char H, unsigned char R_base, unsigned char N, int num_sectors, unsigned char density);
extern struct d88_sector_t *create_d88_custom_sectors(unsigned char C, unsigned char H, const unsigned char *R, const unsigned char *N, int num_sectors, unsigned char density);
extern int get_d88_image_size(const struct d88_image_t *image);
extern int get_d88_disk_size(const struct d88_disk_t *disk);
extern struct d88_track_t *get_d88_track(struct d88_disk_t *disk, int C, int H);
extern struct d88_sector_t *get_d88_sector(struct d88_disk_t *disk, int C, int H, int R);

#endif	/* __D88IMAGE_H__ */
